# Tunnel层API设计
理论上说，传输层的所有抽象都可以基于Tunnel层(网络层)提供的API来实现。但是目前从IP网络的实际情况来考虑，Connection抽象可以单独使用TCP协议来实现，不需要通过我们的网络层API。

## 端到端发送数据

```
int updatePeerInfo(peerid,peerInfo);//配置peerid与peerinfo的关系，尤其是更新peerinfo的可变部分
int sendto(localPeerid,peerid,data);//向peerid发送数据报。（不可靠）
//事件
void onRecvData(localPeerid,remotePeerid,data);
void onPeerInfoUpdate(peerid,peerInfo);

//下面是和目前实现相关的接口，可以获得到目标peerid的tunnel状况
int getTunnelStatus(localPeerid,peerid);//LN到RN之间是否有可通信的tunnel
vector<tunnel> getTunnelContainer(localPeerid,peerid);//可以遍历已经存在的Tunnel,Tunnel对象的核心是一个pair<ep,ep>
//事件
void onTunnelStatusChanged(tunnelid,oldstatus,newstatus);
```

## 广播发送数据(一对多发送)
广播的接口的核心在于group的设计，目前我们支持基于peerid的地域group，基于peerid设计的“某人的设备”group,基于物理层实现的物理group（比如IP协议中的同内网），和基于应用的应用group.
目前应用group的维护接口还是需要详细设计的。不管是哪种group,都可以通过全网唯一的groupid来标识。
```
int broadcast(localPeerid,groupid,data,flags);//向指定的group广播数据报文,flags里可以指定该次广播是否是可靠广播
void onBoradcast(localPeerid,groupid,data,srcPeerID,srcPeerInfo)
//应用级别的组管理接口类似下面
groupid createGroup(localpeerid,groupflags);
int joinGroup(groupid,peerid);
int leaveGroup(groupid,peerid;
```

广播发送还存在一个特殊的场景是broadcast data to any device in group.即可靠的广播数据到组中的任意一个设备，即算广播成功。我在考虑这个特性是放在接口层的flags里号还是做为group特性放到groupid里好。


### 多对多发送数据
目前还没有特别api设计可以用。初步看起来和普通的广播接口是一样的，不同在于任何设备加入组之后都可以直接broadcast。所有人都能收到onBroadcast事件。
这块的接口和buckycloud的global event的设计有相似之处，实现的时候面对的问题也有相似。

### 网络层广播与物理介质实现的关系(TODO)
现在的网络协议（IP协议）是设计在端到端的电缆上工作的，广播工作要么是遍历端口（本质上还是端到端），要么就是所有电缆上传输所有数据（Hub)。所以IP协议的广播功能是很薄弱的。如何利用无线网络（5G）的特性，来更好的设计网络层协议，包括mesh的软件逻辑，都是BDT未来重要的研究课题。


# Session层(传输层)API设计
核心目标是建立更易于使用的抽象供给应用层使用，另一个目标在于在于实现传输层与网络层的正交分层，但是目前来看很多传输层设计是绑定网络层语义的，正交分层的目标并不一定能完全实现。

## Connection（端到端的顺序可靠流数据传输）

```
connection createConnection(localPeerid,remotePeerid,vport,options);
int connection.openAndSend(data,options);
int connection.send(data);
int connection.close();
int connection.free();//注意区别free和close语义。这个语义区别是从tcp时代传下来的，也许只有free会简单。
int connection.onRecv(data);
int connection.onEvent(oldstate,newstate,params);

acceptor createAcceptor(localpeerid,vport,options);
int acceptor.startListen();
int acceptor.onAccept(newConn,firstData);//和openAndSend对应，accept的时候就可以拿到第一份数据
```

## DataChannel (端到端的NamedData传输)
DataChannel的语义来自NDN理论。简单来说就是一个“只关注可校验数据的持有么，不关注主机的”结构。由于不关注主机地址，所以理论上是可以运行在网络层的。

### NDN(面向NamedData传输而不关注端)
NDN的语义更多的是主机无关的。但和广播一样，主机无关的语义不方便做精确控制
如果NDN的API语义层没有“向特点主机开始传输NamedData”的语义，那么其语义控制只有
“我想要某个NameData、这个NameData我已经有了、我不要某个NameData"
“”XXX想要某个NamedData、允许NameDataManager发送数据给他、不再允许发送NameData给他”
“收到了XXX对于某个NamedData的状态变化、是否需要广播出去”
将NameData发送给XXX的实现细节隐藏在NamedDataMananger中，NamedDataManager的逻辑为读取、写入NamedData.Encode/Deocde NamedData.
这种设计可能会难以控制传输细节，具体实现的时候也许是靠上一节的API来实现的。

### 主要流程与API设计
NDN的理论比较传统，我们在引入喷泉码后变得更加纯粹。
1. 主动请求数据
LN->RN:interest_list
RN->LN:push_piece
LN->RN:update interest_list(比如传输完成)

2. 推送数据
RN->LN:push_piece
LN->RN:update interest_list(我需要、我不需要)   
对RN的push拥塞控制算法来说，可以等LN进行了初始回复后才开始全力传输   
DataChannel相关协议为了方便透明加速识别，通常都是明文的，毕竟 NamedData可以进行校验。    
如果LN的同一个NamedData同时有多个RN在push_piece,那么LN需要进行一些调度。   
DataChannel不包含DataTracker服务。DataTracker服务应该是和服务相关的。    

一个Peer上同时只能有一个NamedData的一个下载过程，基于以上设计，相关API设计是很单纯的：
```
int interest(namelist,peerid|groupid);//请求数据,可以指定请求数据时的优先服务器和组
int push(namelist,peerid|groupid);//向特定的端推送数据
int resume(name);
int pause(name);
int getStatus(name);
int setNamedDataManager(namedDataManager);//全局数据管理器

int onRequestNameData(name);//收到请求
int onAcceptNameData(name);//收到推送，在这个事件中可以控制是否开始传输
int onNamedDataTransStatusChanged(name,oldStauts,newStatus);//一个NameData完成了传输


//下面4个是低阶的基础函数，在NDN理论中属于网络层。
int updateInterestList(localpeerid,peerid|groupid,namelist);
int onRecvInterestList(peerid|groupid,interestlist);
int pushPiece(peerid|groupid,piece)
int onRecvPiece(peerid|groupid,piece)//具体实现时可以有push_resp来进行拥塞控制。

```

### 顺序请求NamedData(分层传输)
分层传输的概念如下:   
原计划传输NamedData N.这个N接收端只有接收完了才有用。现在把N拆分成N1,N2,N3... 拆分出来的NamedData的大小依次增大。使用者能阶段性的获得N1,N2,N3...并使用这些数据。这里涉及的新问题就是NamedData的粒度（大小）：NamedData代表的一块data block不能太小了，否则如何传输Name本身又成问题（元数据比例问题）     
也许对于NamedData不能太小的限制，会影响NDN理论的应用场景。（HTTP小文件无法使用P2P加速的原理类似）。    

从API设计的角度来说有几种实现思路(看起来b相对靠谱)    
a.LN按顺序请求nameddata，RN无需特别处理    
b.发送带过期时间的namelist，按namelist约定进行顺序push    
c.在name中引入依赖机制。    



### DataChannel的拥塞控制
从NDN基础理论的角度来看，push_piece的过程是非链接性的，这个特性会让速度控制（拥塞控制）的粒度变的很粗（比如主机级的），我们可能需要基于name级的拥塞控制，但这又会导致粒度可能过大(nameddata代表的data block比较小的话）。我们需要抽象的DataChannel，就可以给一个更合理的控制粒度。这个channel的概念，也可以用在分层传输之上。    



### Piece的可Cache性
大部分情况下,都不应该缓存Piece，而是缓存NamedData.但是一些对传输延时敏感的环境下，需要缓存Piece，那么就需要遵循下列准则：    
a.知道缓存的Piece是谁创建发给谁的    
b.原则上只能cache(转发)发送给自己的piece.    


### 溢出传输与NamedData大小
基于NamedData的传输一定会存在流量浪费，一个RN在给LN push_piece,当LN已经收到NamedData,广播已接收完成需要的时间为RTT，那么在这个事件里，所有RN给LN push的piece都是浪费掉的。如果LN接收piece的速度越快，RTT越大，那么浪费掉的字节就越多。如果NamedData还较小，那么整个网络浪费的字节比例可能会很大。要减少这种浪费，可以在小NamedData上引入一定的ACK机制，对传输LN的传输进度进行实时通报，让RN有机会提前结束piece push.

`未来网速提升，控制指令所需要的延迟带来的浪费问题肯定会越来越严重`。

## 广播NamedData
广播NameData计划用于视频会议场景，分层发送package好的视频数据和音频数据。
现在有追求极限低延迟的视频直播场景，可能不适合用NamedData场景（源一帧一帧的推数据，只向前reference压缩），基本属于信源发多少，接收端努力收多少的情况。


# 结构化传输模型
今天的互联网大量的应用服务都是基于结构化数据的，我们可能需要设计一些专用的传输API，来让这些服务工作的更好。这部分设计的不少根源来自我们在BuckyCloud研发过程中的实践。由于短期内不会涉及，这里只是提一下，具体就不展开了。
## QA(结构化请求)
## GlobalEvent


#  作业：基于上述抽象的一些简单但创新的App例子
新的传输API应该能有助于未来去中心化APP的开发（去中心化APP不会只依靠BDT就能完成开发），我这里先举一个去中心化微信的例子，大家可以思考如何使用BDT帮助更好的实现其它的典型应用。   

## 去中心微信
### 个人服务器（POD）：
1.客户端启动后先加载本地缓存，再连接POD
2.连接成功可以同步POD中收到的MSG（MSG引用的图片、视频都存放在POD的filesystem中）
3.POD会自动地把相关数据（结构化和非结构化数据）加密后备份到ffs上
4.新的POD会尝试在FFS的备份中重建数据。
5.客户端和POD能力是相近的，只是POD的在线时间更长
6.如果客户没有自己的POD，则可以租用一台POD。这台POD也会执行步骤3，这样用户在感觉POD租用平台不靠谱的情况下，可以完全可靠的创建自己的POD
7.即没有自己的POD，也不租用POD的情况下，用户是没办法使用新的微信的。（POD是必须的）

### 发送文本消息：
1.客户端给客户端发消息：广播msg给“朋友的所有设备”
2.接收客户端解析MSG，显示，并同步MSG中的named data
3.发送端只会收到POD的广播msg确认。确保信息已经正确发送了。

### 在沙漠中发送消息：
沙漠中的设备自动组成了一个基于广播的对等网络，或则在人工干预的情况下，组成了一个星形网络
BDT的广播功能可以在这个网络中使用，上述功能得到恢复。

### 实时多人视频通话：
基本路径： 客户端采集音视频->分层编码->向临时Group广播->分层解码->回放
如果客户端的计算能力不够，且到POD的带宽稳定快速，那么可以变成：
客户端采集音视频->传输到POD->分层编码->向临时Group广播->POD收到数据->分层解码->传输到客户端->回放
POD加入后，可以轻松支持 语言会议、视频会议的录制功能。

#### 参加会议的人特别多的时候:
在广域网上（肯定是基于有线的）不能所有人都成为第一层接收数据的对象了，可能需要进行分级广播。
基于网络编码（喷泉码）的NamedData广播结构


## 视频会议
## 云游戏
## 新的HTTP服务器
## 个人文件系统(NAS)

# 其它

## 与serverless(智能合约的关系)
在编写代码的时候可以不写任何PeerID,而是由部署和运行的时候决定。

## 兼容与过渡
企业级POD，可以运行一些传统的企业应用（比如电子邮件）。


