# 可信设备间数据传输
已经建成的IP网络是人类构建的最可靠的基础设施之一。随着软硬件的不断发展，越来越多样的设备通过IP协议接入互联网。当下应用最常见的需求是：和某个特定的设备通信；在IP网络里，对设备的标识是其IP地址，应用要定义身份认证逻辑来关联IP地址和可信的设备；并且IP(IPV6尚未普及)地址因为IPV4映射不能保证所有IP地址可以互相联通，一般还需要应用通过固定公网地址的中心服务器中转；

以智能家居场景为例，用手机通过IP网络向扫地机器人发送一条开始清扫的命令，需要编写如下代码：
```
手机端:
建立到控制服务器的连接(控制服务器IP地址已知);
发送登录指令，等待控制服务器确认身份;
登录成功，该链接为可信链接;
通过该链接向控制服务器发送“开始清扫”指令;
```

```
扫地机器人端:
建立到控制服务器的连接（控制服务器IP地址已知);
发送登录指令，等等控制服务器确认身份;
登录成功，等待指令
收到控制服务器转发的“开始清扫”指令;
```

BDT协议被设计用来在两个可信设备之间进行通信，实现如下语义：
1. 基于设备标识而非IP地址来构建网络。
2. 在任意设备上可以无中心的认证其他设备的身份。
3. 任意设备和设备之间可以直接传输，加密的，可信的数据。

基于BDT协议，上面扫地机器人的应用，实现代码如下：
```
手机端：
通过扫地机器人的设备标识，建立到扫地机器人的加密连接;
连接建立成功，发送“开始清扫指令”;
```

```
扫地机器人端:
收到连接建立请求，读取请求端设备标识;
确认请求端的设备标识有权限控制自己，加密连接建立成功;
通过加密连接收到手机端发送的“开始清扫”指令;
```

## Device Object:可认证的设备标识
BDT用Named Object系统设计的Device Object来标识设备;
命名对象(Named Object)系统源自区块链，命名对象是结构化的信息。我们可以用json来定义一个类型为“Device”的命名对象：
```json
{
    "area" : "china",
    "device_type" : "android",
    "device_id" : "A23B7A3", 
}
```
和这个json语义一致的命名对象称作普通命名对象，这类对象是`不可变对象`。命名对象的编码标准也是系统定义的一部分，对编码结果进行Hash计算得到该命名对象的`ObjectId`。语义上相同的命名对象编码后的结果相同，所以也有着相同的`ObjectId`.
```
ObjectId = Hash(Encode(NamedObject))
```
接下来，我们在这个对象中，增加一个重要的字段public_key,指向一对非对称加密算法生成的密钥中的共钥。同时创建这个对象的自然人，自然会妥善的保存其私钥。这种包含有公钥的命名对象被称作`有权对象`。能通过签名的方法进行`授权`。
```json
{
    "public_key":"xxxxxx",
    "area" : "china",
    "device_type" : "android",
    "device_id" : "A23B7A3", 
}
```

## Tunnel: 设备之间的可信数据通道
基于这样的设备标识,建立可信通信信道Tunnel的逻辑过程如下:
```
1.device1 生成临时的，对称密钥K, 通过链路层协议向device2发送握手包；握手包包含K，用device2的public_key加密,并使用device1的private key签名；
2.device2收到握手包后，使用自己secrect解密握手包，记录K。随后回复使用K加密的ACK包；
3.device1和device2 之间使用K加密后续数据包通信；
```

BDT使用“多子网注册/发布” 机制，允许Internet上存在更多的，不同类型的链路层协议。IP协议从传输层协议下降为链路的一种实现，同时支持TCP/UDP,IPv4/IPv6，还内建支持通过其他节点中转的代理链路；应用还可以通过一定的策略，选择合适的IP链路实现Tunnel的链路优化；当然BDT还需要解决传统P2P的NAT穿透等问题：有固定外网地址的Super Node节点参与进来。

## 应用层网络语义抽象
基于IP协议的应用编程，很少直接收发IP数据报，而是使用传输层协议TCP/UDP; 基于BDT协议的应用变成，同样很少直接使用Tunnel收发数据报；BDT内建实现了Tunnel之上的应用层语义对象Stream（TCP）和Datagram（UDP）；

### Stream: 
提供了全双工，可靠数据传输语义。
+ 提供了在握手包嵌入轻量Q&A的FastQA能力。
+ 定制拥塞控制算法。

### Datagram:
提供了无连接的不可靠数据报收发语义。
+ 超过链路层MTU的能力；
+ 在广域网广播的能力；


# 命名数据传输
如果接收端明确知道要接收的数据的数字摘要，就可以通过摘要来校验收到数据，这个场景下并不需要从可信设备接收数据，因为不惧怕中间人篡改数据，也不用认证发送者的身份。这些数据的传输可以非加密的，并且来源无关的。当前互联网传播的流量大部分是这样的数据。比如，一部电影的内容。

命名数据网络理论 定义网络中的终端和中继节点，发送/缓存/丢弃/转发 Interest 和 Data 两类基础Packet的行为，实现命名数据在网络中的高效传输。
一个典型的场景是：同一路由器R的两设备D1和D2上，同时播放一部热门的电影，D1和D2向影片服务器S同时请求相同的数据D；原本从S到R之间的链路上的下行流量是2*sizeof(D); 在NDN网络理论下，R的行为可以是：
+ 向S转发D1对D的请求；
+ 缓存从S发向D1的D；
+ 丢弃D2向S发送的对D的请求；
+ 从R向D2发送数据D；
此时从事过S到R之间的链路上的下行流量变为sizeof(D);达到了优化网络整体流量的效果。

BDT协议也支持NDN理论传输命名数据：
+ Tunnel上收发明文数据报的能力；
+ 引入Chunk数据类型来定义一段命名对象；
+ 引入Channel语义，基于明文tunnel来实现chunk在device之间的传输，channel可以有不同于stream的编码模式，发送窗口控制，以及拥塞控制算法。
+ 引入Task语义对象，定义NDN理论中Interest/Data Packet, 向网络请求一段/多段 Chunk；

# 完全的去中心
+ Device Object引入PoW信用
+ DHT 
+ 网络中的服务节点：服务证明和激励结算支持，比如 SuperNode，ProxyNode， NDN网络中的路由，缓存，索引节点；


